"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const Bridge_1 = require("./Bridge");
const Stream_1 = require("./Stream");
const utils_1 = require("./utils");
class BridgePlus extends Bridge_1.Bridge {
    /**
     * @external
     */
    static init() {
        super.init();
        this.onMessage('__crx_bridge_stream_open__', (message) => {
            return new Promise((resolve) => {
                const { sender, data } = message;
                const { channel } = data;
                let watching = false;
                const readyup = () => {
                    const callback = this.onOpenStreamCallbacks.get(channel);
                    if (typeof callback === 'function') {
                        callback(new Stream_1.Stream({ ...data, endpoint: sender }));
                        if (watching) {
                            this.streamyEmitter.removeListener('did-change-stream-callbacks', readyup);
                        }
                        resolve(true);
                    }
                    else if (!watching) {
                        watching = true;
                        this.streamyEmitter.on('did-change-stream-callbacks', readyup);
                    }
                };
                readyup();
            });
        });
    }
    static onOpenStreamChannel(channel, callback) {
        if (this.onOpenStreamCallbacks.has(channel)) {
            throw new Error(`crx-bridge: This channel has already been claimed. Stream allows only one-on-one communication`);
        }
        this.onOpenStreamCallbacks.set(channel, callback);
        this.streamyEmitter.emit('did-change-stream-callbacks');
    }
    static async openStream(channel, destination) {
        if (this.openStreams.has(channel)) {
            throw new Error(`crx-bridge: A Stream is already open at this channel`);
        }
        const streamInfo = {
            streamId: utils_1.uuid(),
            channel,
            endpoint: destination,
        };
        const stream = new Stream_1.Stream(streamInfo);
        stream.onClose(() => {
            this.openStreams.delete(channel);
        });
        await this.sendMessage('__crx_bridge_stream_open__', streamInfo, destination);
        this.openStreams.set(channel, stream);
        return stream;
    }
}
BridgePlus.openStreams = new Map();
BridgePlus.onOpenStreamCallbacks = new Map();
BridgePlus.streamyEmitter = new events_1.EventEmitter();
exports.BridgePlus = BridgePlus;
BridgePlus.init();
//# sourceMappingURL=BridgePlus.js.map